"use strict";
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// index.ts
var index_exports = {};
__export(index_exports, {
  Config: () => Config,
  Context: () => Context,
  Prefab: () => Prefab,
  prefab: () => prefab,
  version: () => version_default
});
module.exports = __toCommonJS(index_exports);

// src/prefab.ts
var import_uuid = require("uuid");

// src/config.ts
var parseRawMetadata = (metadata) => ({
  configRowIndex: parseInt(metadata.configRowIndex, 10),
  conditionalValueIndex: parseInt(metadata.conditionalValueIndex, 10),
  type: metadata.type,
  configId: metadata.id
});
var valueFor = (value, type, key) => {
  const rawValue = value[type];
  switch (type) {
    case "json":
      try {
        return JSON.parse(rawValue);
      } catch (e) {
        console.error(`Error parsing JSON from Prefab config ${key}`, e, rawValue);
        return value[type];
      }
    case "duration": {
      const duration = rawValue;
      return {
        ms: duration.millis,
        seconds: duration.millis / 1e3
      };
    }
    default:
      return rawValue;
  }
};
var parseEvaluationPayload = (payload) => {
  const configs = {};
  Object.keys(payload.evaluations).forEach((key) => {
    const evaluation = payload.evaluations[key];
    const type = Object.keys(evaluation.value)[0];
    configs[key] = new Config(
      key,
      valueFor(evaluation.value, type, key),
      type,
      evaluation.value,
      evaluation.configEvaluationMetadata ? parseRawMetadata(evaluation.configEvaluationMetadata) : void 0
    );
  });
  return configs;
};
var parseRawConfigWithoutTypes = (payload) => {
  const configs = {};
  Object.keys(payload).forEach((key) => {
    const type = typeof payload[key];
    configs[key] = new Config(key, valueFor({ [type]: payload[key] }, type, key), type);
  });
  return configs;
};
var Config = class {
  constructor(key, value, type, rawValue, metadata) {
    this.key = key;
    this.value = value;
    this.type = type;
    this.rawValue = rawValue;
    this.configEvaluationMetadata = metadata;
  }
  static digest(payload) {
    if (payload === void 0) {
      console.trace("Config.digest called with undefined payload");
    }
    if ("evaluations" in payload) {
      return parseEvaluationPayload(payload);
    }
    return parseRawConfigWithoutTypes(payload);
  }
};

// src/base64Encode.ts
var base64Encode = (obj) => {
  if (typeof window !== "undefined") {
    if (typeof TextEncoder === "undefined") {
      return window.btoa(obj);
    }
    const bytes = new TextEncoder().encode(obj);
    const binString = Array.from(bytes, (byte) => String.fromCodePoint(byte)).join("");
    return btoa(binString);
  }
  return Buffer.from(obj).toString("base64");
};
var base64Encode_default = base64Encode;

// src/context.ts
var isEqual = (a, b) => {
  const aKeys = Object.keys(a);
  const bKeys = Object.keys(b);
  if (aKeys.length !== bKeys.length) {
    return false;
  }
  return aKeys.every((key) => {
    const aValues = a[key];
    const bValues = b[key];
    const aValuesKeys = Object.keys(aValues);
    const bValuesKeys = Object.keys(bValues);
    if (aValuesKeys.length !== bValuesKeys.length) {
      return false;
    }
    return aValuesKeys.every((ckey) => {
      const aValue = aValues[ckey];
      const bValue = bValues[ckey];
      return aValue === bValue;
    });
  });
};
var getType = (value) => {
  if (typeof value === "string") {
    return "string";
  }
  if (typeof value === "number") {
    if (Number.isInteger(value)) {
      return "int";
    }
    return "double";
  }
  return "bool";
};
var Context = class {
  constructor(contexts) {
    if (!Object.values(contexts).every((item) => typeof item === "object")) {
      console.error("Context must be an object where the value of each key is also an object");
    }
    if (Object.values(contexts).some(
      (item) => Object.values(item).some((value) => typeof value === "object" && value !== null)
    )) {
      console.error("Nested objects are not supported in context values at this time");
    }
    this.contexts = contexts;
  }
  equals(other) {
    return isEqual(this.contexts, other.contexts);
  }
  encode() {
    const formatted = Object.keys(this.contexts).map((key) => {
      const values = {};
      Object.keys(this.contexts[key]).forEach((ckey) => {
        values[ckey] = {
          [getType(this.contexts[key][ckey])]: this.contexts[key][ckey]
        };
      });
      return {
        type: key,
        values
      };
    });
    return encodeURIComponent(base64Encode_default(JSON.stringify({ contexts: formatted })));
  }
};

// src/exponentialBackoff.ts
var ExponentialBackoff = class {
  // arguments are in seconds
  constructor(maxDelay, initialDelay = 2, multiplier = 2) {
    this.initialDelay = initialDelay;
    this.maxDelay = maxDelay;
    this.multiplier = multiplier;
    this.delay = initialDelay;
  }
  call() {
    const delayValue = this.delay;
    this.delay = Math.min(this.delay * this.multiplier, this.maxDelay);
    return delayValue * 1e3;
  }
};

// src/periodicSync.ts
var PeriodicSync = class _PeriodicSync {
  constructor(client, name, syncInterval) {
    this.data = /* @__PURE__ */ new Map();
    this.client = client;
    this.name = name;
    this.startAt = /* @__PURE__ */ new Date();
    this.startPeriodicSync(syncInterval);
  }
  stop() {
    clearTimeout(this.timeoutID);
  }
  sync() {
    if (this.data.size === 0) return;
    this.logInternal(`${this.name} syncing ${this.data.size} items`);
    const startAtWas = this.startAt;
    this.startAt = /* @__PURE__ */ new Date();
    this.flush(this.prepareData(), startAtWas);
  }
  prepareData() {
    const toShip = new Map(this.data);
    this.data.clear();
    return toShip;
  }
  startPeriodicSync(syncInterval) {
    this.startAt = /* @__PURE__ */ new Date();
    this.syncInterval = _PeriodicSync.calculateSyncInterval(syncInterval);
    this.scheduleNextSync();
  }
  scheduleNextSync() {
    const interval = this.syncInterval();
    this.timeoutID = setTimeout(() => {
      this.sync();
      this.scheduleNextSync();
    }, interval);
  }
  static calculateSyncInterval(syncInterval) {
    if (syncInterval !== void 0) {
      return () => syncInterval;
    }
    const backoff = new ExponentialBackoff(60 * 5, 8);
    return () => backoff.call();
  }
  logInternal(message) {
    const loggerName = `${this.client.clientNameString}.prefab.${this.name}`;
    if (this.client.shouldLog(
      {
        loggerName,
        desiredLevel: "debug",
        defaultLevel: "error"
      },
      false
      // synchronous so that this log ends up in the current batch
    )) {
      console.log(`${loggerName}: ${message}`);
    }
  }
};

// src/evaluationSummaryAggregator.ts
var massageSelectedValue = (config) => {
  if (config.rawValue && (config.type === "duration" || config.type === "json")) {
    if (config.type === "json") {
      return { json: config.rawValue[config.type] };
    }
    return config.rawValue[config.type];
  }
  return config.type === "stringList" ? { values: config.value } : config.value;
};
var massageConfigForTelemetry = (config, metadata) => __spreadProps(__spreadValues({}, metadata), {
  selectedValue: {
    [config.type]: massageSelectedValue(config)
  },
  count: 0
});
var EvaluationSummaryAggregator = class _EvaluationSummaryAggregator extends PeriodicSync {
  constructor(client, maxKeys, syncInterval) {
    super(client, "EvaluationSummaryAggregator", syncInterval != null ? syncInterval : 3e4);
    this.maxKeys = maxKeys;
  }
  record(config) {
    if (this.data.size >= this.maxKeys) return;
    if (config == null ? void 0 : config.configEvaluationMetadata) {
      const _a = config.configEvaluationMetadata, { type } = _a, metadata = __objRest(_a, ["type"]);
      const key = `${config.key},${type}`;
      if (!this.data.has(key)) {
        this.data.set(key, massageConfigForTelemetry(config, metadata));
      }
      const counter = this.data.get(key);
      if (counter) {
        counter.count += 1;
      }
    }
  }
  flush(toShip, startAtWas) {
    var _a;
    const summaries = {
      start: startAtWas.getTime(),
      end: (/* @__PURE__ */ new Date()).getTime(),
      summaries: _EvaluationSummaryAggregator.summaries(toShip)
    };
    (_a = this.client.telemetryUploader) == null ? void 0 : _a.post(this.events(summaries));
  }
  static summaries(data) {
    return Array.from(data).map((entry) => {
      const [configKey, configType] = entry[0].split(",");
      const counter = entry[1];
      const counters = [counter];
      return {
        key: configKey,
        type: configType,
        counters
      };
    });
  }
  events(summaries) {
    const event = { summaries };
    return {
      instanceHash: this.client.instanceHash,
      events: [event]
    };
  }
};

// src/apiHelpers.ts
var headers = (apiKey, clientVersion) => ({
  Authorization: `Basic ${base64Encode_default(`u:${apiKey}`)}`,
  "X-PrefabCloud-Client-Version": clientVersion
});
var DEFAULT_TIMEOUT = 1e4;

// src/loader.ts
var defaultEndpoints = ["belt", "suspenders", "waistband"].map(
  (subdomain) => `https://${subdomain}.prefab.cloud/api/v2`
);
var EARLY_TIMEOUT = 2e3;
var Loader = class {
  constructor({
    apiKey,
    context,
    endpoints = void 0,
    timeout,
    collectContextMode = "PERIODIC_EXAMPLE",
    clientVersion = ""
  }) {
    this.collectContextMode = "PERIODIC_EXAMPLE";
    this.isAborted = false;
    this.apiKey = apiKey;
    this.context = context;
    this.endpoints = endpoints || defaultEndpoints;
    this.timeout = timeout || DEFAULT_TIMEOUT;
    this.collectContextMode = collectContextMode;
    this.clientVersion = clientVersion;
  }
  url(root) {
    return `${root}/configs/eval-with-context/${this.context.encode()}?collectContextMode=${this.collectContextMode}`;
  }
  loadFromEndpoint(index, options, resolve, reject) {
    this.abortController = new AbortController();
    const { signal } = this.abortController;
    this.isAborted = false;
    const endpoint = this.endpoints[index];
    const url = this.url(endpoint);
    fetch(url, __spreadValues({ signal }, options)).then((response) => {
      this.clearAbortTimeout();
      if (response.ok) {
        return response.json();
      }
      throw new Error(`${response.status} ${response.statusText}`);
    }).then((data) => {
      if (!("evaluations" in data)) {
        throw new Error(`Invalid payload:${JSON.stringify(data)}`);
      }
      resolve(data);
    }).catch((error) => {
      this.clearAbortTimeout();
      if (index < this.endpoints.length - 1) {
        this.loadFromEndpoint(index + 1, options, resolve, reject);
      } else {
        reject(error);
      }
    });
    const timeout = index < this.endpoints.length - 1 ? Math.min(this.timeout, EARLY_TIMEOUT) : this.timeout;
    this.abortTimeoutId = setTimeout(() => {
      var _a;
      if (!this.isAborted) {
        this.isAborted = true;
        (_a = this.abortController) == null ? void 0 : _a.abort();
      }
    }, timeout);
  }
  load() {
    var _a;
    if (!this.isAborted) {
      this.isAborted = true;
      (_a = this.abortController) == null ? void 0 : _a.abort();
    }
    const options = {
      headers: headers(this.apiKey, this.clientVersion)
    };
    const promise = new Promise((resolve, reject) => {
      this.loadFromEndpoint(0, options, resolve, reject);
    });
    return promise;
  }
  clearAbortTimeout() {
    clearTimeout(this.abortTimeoutId);
  }
};

// src/logger.ts
var PREFIX = "log-level";
var WORD_LEVEL_LOOKUP = {
  TRACE: 1,
  DEBUG: 2,
  INFO: 3,
  WARN: 5,
  ERROR: 6,
  FATAL: 9
};
var isValidLogLevel = (logLevel) => Object.keys(WORD_LEVEL_LOOKUP).includes(logLevel.toUpperCase());
var shouldLog = ({
  loggerName,
  desiredLevel,
  defaultLevel,
  get
}) => {
  let loggerNameWithPrefix = `${PREFIX}.${loggerName}`;
  const desiredLevelNumber = WORD_LEVEL_LOOKUP[desiredLevel.toUpperCase()];
  while (loggerNameWithPrefix.length > 0) {
    const resolvedLevel = get(loggerNameWithPrefix);
    if (resolvedLevel !== void 0) {
      return WORD_LEVEL_LOOKUP[resolvedLevel.toString()] <= desiredLevelNumber;
    }
    if (loggerNameWithPrefix.indexOf(".") === -1) {
      break;
    }
    loggerNameWithPrefix = loggerNameWithPrefix.slice(0, loggerNameWithPrefix.lastIndexOf("."));
  }
  return WORD_LEVEL_LOOKUP[defaultLevel.toUpperCase()] <= desiredLevelNumber;
};

// src/telemetryUploader.ts
var TelemetryUploader = class _TelemetryUploader {
  constructor({
    apiKey,
    apiEndpoint = void 0,
    timeout,
    clientVersion
  }) {
    this.apiKey = apiKey;
    this.apiEndpoint = apiEndpoint || "https://telemetry.prefab.cloud/api/v1";
    this.timeout = timeout || DEFAULT_TIMEOUT;
    this.clientVersion = clientVersion;
  }
  clearAbortTimeout() {
    clearTimeout(this.abortTimeoutId);
  }
  static postUrl(root) {
    return `${root}/telemetry`;
  }
  postToEndpoint(options, resolve, reject) {
    const controller = new AbortController();
    const signal = controller == null ? void 0 : controller.signal;
    let isAborted = false;
    const url = _TelemetryUploader.postUrl(this.apiEndpoint);
    fetch(url, __spreadValues({ signal }, options)).then((response) => {
      this.clearAbortTimeout();
      if (response.ok) {
        return response.json();
      }
      console.warn(
        `Prefab warning: Error uploading telemetry ${response.status} ${response.statusText}`
      );
      return response.status;
    }).then((response) => {
      resolve(response);
    }).catch((error) => {
      this.clearAbortTimeout();
      if (error.name === "AbortError") {
        try {
          console.debug("Prefab telemetry request aborted");
        } catch (e) {
        }
        resolve({ status: "aborted" });
        return;
      }
      reject(error);
    });
    this.abortTimeoutId = setTimeout(() => {
      if (!isAborted) {
        isAborted = true;
        controller.abort();
      }
    }, this.timeout);
  }
  post(data) {
    const options = {
      method: "POST",
      headers: __spreadProps(__spreadValues({}, headers(this.apiKey, this.clientVersion)), {
        "Content-Type": "application/json",
        Accept: "application/json"
      }),
      body: JSON.stringify(data),
      keepalive: true
      // needed for flushing when the window is closed
    };
    const promise = new Promise((resolve, reject) => {
      this.postToEndpoint(options, resolve, reject);
    });
    return promise;
  }
};

// src/loggerAggregator.ts
var SEVERITY_KEY = {
  TRACE: "traces",
  DEBUG: "debugs",
  INFO: "infos",
  WARN: "warns",
  ERROR: "errors",
  FATAL: "fatals"
};
var LoggerAggregator = class extends PeriodicSync {
  constructor(client, maxLoggers, syncInterval) {
    super(client, "LoggerAggregator", syncInterval != null ? syncInterval : 3e4);
    this.maxLoggers = maxLoggers;
  }
  record(logger, level) {
    if (this.data.size >= this.maxLoggers) return;
    if (!this.data.has(logger)) {
      this.data.set(logger, {
        loggerName: logger,
        traces: 0,
        debugs: 0,
        infos: 0,
        warns: 0,
        errors: 0,
        fatals: 0
      });
    }
    const counter = this.data.get(logger);
    if (counter) {
      const severityKey = SEVERITY_KEY[level];
      counter[severityKey] += 1;
    }
  }
  flush(toShip, startAtWas) {
    var _a;
    const loggers = {
      startAt: startAtWas.getTime(),
      endAt: (/* @__PURE__ */ new Date()).getTime(),
      loggers: Array.from(toShip.values())
    };
    (_a = this.client.telemetryUploader) == null ? void 0 : _a.post(this.events(loggers));
  }
  events(loggers) {
    const event = { loggers };
    return {
      instanceHash: this.client.instanceHash,
      events: [event]
    };
  }
};

// src/version.ts
var version_default = "0.4.5";

// src/prefab.ts
var Prefab = class {
  constructor() {
    this._configs = {};
    this._pollCount = 0;
    this._pollStatus = { status: "not-started" };
    this._pollTimeoutId = void 0;
    this._instanceHash = (0, import_uuid.v4)();
    this.collectEvaluationSummaries = true;
    this.collectLoggerNames = false;
    this.clientNameString = "prefab-cloud-js";
    this.loaded = false;
    this.afterEvaluationCallback = () => {
    };
    this._context = new Context({});
  }
  init(_0) {
    return __async(this, arguments, function* ({
      apiKey,
      context: providedContext,
      endpoints = void 0,
      apiEndpoint,
      timeout = void 0,
      afterEvaluationCallback = () => {
      },
      collectEvaluationSummaries = true,
      collectLoggerNames = false,
      collectContextMode = "PERIODIC_EXAMPLE",
      clientNameString = "prefab-cloud-js",
      clientVersionString = version_default
    }) {
      const context = providedContext != null ? providedContext : this.context;
      if (!context) {
        throw new Error("Context must be provided");
      }
      this._context = context;
      this.clientNameString = clientNameString;
      const clientNameAndVersionString = `${clientNameString}-${clientVersionString}`;
      this.loader = new Loader({
        apiKey,
        context,
        endpoints,
        timeout,
        collectContextMode,
        clientVersion: clientNameAndVersionString
      });
      this._telemetryUploader = new TelemetryUploader({
        apiKey,
        apiEndpoint,
        timeout,
        clientVersion: clientNameAndVersionString
      });
      this.collectEvaluationSummaries = collectEvaluationSummaries;
      if (collectEvaluationSummaries) {
        this.evalutionSummaryAggregator = new EvaluationSummaryAggregator(this, 1e5);
      }
      this.collectLoggerNames = collectLoggerNames;
      if (collectLoggerNames) {
        this.loggerAggregator = new LoggerAggregator(this, 1e5);
      }
      if ((collectEvaluationSummaries || collectLoggerNames) && typeof window !== "undefined" && typeof window.addEventListener === "function") {
        window.addEventListener("beforeunload", () => {
          var _a, _b;
          (_a = this.evalutionSummaryAggregator) == null ? void 0 : _a.sync();
          (_b = this.loggerAggregator) == null ? void 0 : _b.sync();
        });
      }
      this.afterEvaluationCallback = afterEvaluationCallback;
      return this.load();
    });
  }
  get configs() {
    return this._configs;
  }
  get context() {
    return this._context;
  }
  get instanceHash() {
    return this._instanceHash;
  }
  get pollTimeoutId() {
    return this._pollTimeoutId;
  }
  get pollCount() {
    return this._pollCount;
  }
  get pollStatus() {
    return this._pollStatus;
  }
  get telemetryUploader() {
    return this._telemetryUploader;
  }
  load() {
    return __async(this, null, function* () {
      if (!this.loader || !this.context) {
        throw new Error("Prefab not initialized. Call init() first.");
      }
      if (globalThis && globalThis._prefabBootstrap) {
        const prefabBootstrap = globalThis._prefabBootstrap;
        const bootstrapContext = new Context(prefabBootstrap.context);
        if (this.context.equals(bootstrapContext)) {
          this.setConfig({ evaluations: prefabBootstrap.evaluations });
          return Promise.resolve();
        }
      }
      this.loader.context = this.context;
      return this.loader.load().then((rawValues) => {
        this.setConfig(rawValues);
      }).finally(() => {
        if (this.pollStatus.status === "running") {
          this._pollCount += 1;
        }
      });
    });
  }
  updateContext(context, skipLoad = false) {
    return __async(this, null, function* () {
      if (!this.loader) {
        throw new Error("Prefab not initialized. Call init() first.");
      }
      this._context = context;
      if (skipLoad) {
        return Promise.resolve();
      }
      return this.load();
    });
  }
  poll(_0) {
    return __async(this, arguments, function* ({ frequencyInMs }) {
      if (!this.loader) {
        throw new Error("Prefab not initialized. Call init() first.");
      }
      this.stopPolling();
      this._pollStatus = { status: "pending" };
      return this.loader.load().finally(() => {
        this.doPolling({ frequencyInMs });
      });
    });
  }
  doPolling({ frequencyInMs }) {
    this._pollTimeoutId = setTimeout(() => {
      this.load().finally(() => {
        if (this.pollStatus.status === "running") {
          this.doPolling({ frequencyInMs });
        }
      });
    }, frequencyInMs);
    this._pollStatus = {
      status: "running",
      frequencyInMs
    };
  }
  stopPolling() {
    if (this.pollTimeoutId) {
      clearTimeout(this.pollTimeoutId);
      this._pollTimeoutId = void 0;
    }
    this._pollStatus = { status: "stopped" };
  }
  stopTelemetry() {
    var _a, _b;
    if (this.telemetryUploader) {
      (_a = this.evalutionSummaryAggregator) == null ? void 0 : _a.stop();
      (_b = this.loggerAggregator) == null ? void 0 : _b.stop();
    }
  }
  setConfig(rawValues) {
    this._configs = Config.digest(rawValues);
    this.loaded = true;
  }
  isEnabled(key) {
    return this.get(key) === true;
  }
  get(key) {
    if (!this.loaded) {
      if (!key.startsWith(PREFIX)) {
        console.warn(
          `Prefab warning: The client has not finished loading data yet. Unable to look up actual value for key "${key}".`
        );
      }
      return void 0;
    }
    const config = this.configs[key];
    const value = config == null ? void 0 : config.value;
    if (!key.startsWith(PREFIX)) {
      if (this.collectEvaluationSummaries) {
        setTimeout(() => {
          var _a;
          return (_a = this.evalutionSummaryAggregator) == null ? void 0 : _a.record(config);
        });
      }
      setTimeout(() => this.afterEvaluationCallback(key, value, this.context));
    }
    return value;
  }
  getDuration(key) {
    const value = this.get(key);
    if (!value) {
      return void 0;
    }
    if (!Object.prototype.hasOwnProperty.call(value, "seconds") || !Object.prototype.hasOwnProperty.call(value, "ms")) {
      throw new Error(`Value for key "${key}" is not a duration`);
    }
    return value;
  }
  shouldLog(args, async = true) {
    if (this.collectLoggerNames && isValidLogLevel(args.desiredLevel)) {
      const record = () => {
        var _a;
        return (_a = this.loggerAggregator) == null ? void 0 : _a.record(args.loggerName, args.desiredLevel.toUpperCase());
      };
      if (async) {
        setTimeout(record);
      } else {
        record();
      }
    }
    return shouldLog(__spreadProps(__spreadValues({}, args), { get: this.get.bind(this) }));
  }
  isCollectingEvaluationSummaries() {
    return this.collectEvaluationSummaries;
  }
  isCollectingLoggerNames() {
    return this.collectLoggerNames;
  }
};
var prefab = new Prefab();
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Config,
  Context,
  Prefab,
  prefab,
  version
});
//# sourceMappingURL=index.cjs.map